#!/usr/bin/perl

use strict;
use Date::Calc qw(Add_Delta_Days Days_in_Month);

my $command = "/opt/rt3/local/sbin//statdump.pl";
my $defaultqueue = "All";

my ($opt_m, $opt_w, $opt_d) = (0 , 0, 0);
my $queuename = "";
my $flags = "";

if (defined($ARGV[0])) {
  while ($ARGV[0] ne "") {
    my $arg = $ARGV[0]; shift @ARGV;
    SWITCH: {
      if ($arg =~ /^-m$/) { $opt_m = 1; last SWITCH; }
      elsif ($arg =~ /^-w$/) { $opt_w = 1; last SWITCH; }
      elsif ($arg =~ /^-d$/) { $opt_d = 1; last SWITCH; }
      elsif ($arg =~ /^-(.*)/) {
        if ($flags eq "") {
          $flags = $1;
        } else {
          $flags .= $1;
        }
        last SWITCH;
      }
      else { $queuename = $arg; }
    }
  }
  if ($opt_m + $opt_w + $opt_d == 0) { usage(); exit(1); }
  if ($queuename eq "") { $queuename = $defaultqueue; }; } else {
  usage();
  exit(1);
}

## This is "NOW"
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

if ($opt_m) {
  ## End date is "yesterday"
  my ($end_year, $end_month, $end_day) =
    Add_Delta_Days($year + 1900, $mon + 1, $mday, -1);

  $end_day = Days_in_Month($end_year, $end_month);

  ## Start date is first day of the month
  my ($start_year, $start_month, $start_day) = ($end_year, $end_month, 1);

  invoke_statistics($start_year, $start_month, $start_day, 
                    $end_year, $end_month, $end_day,
                    $flags);
}

if ($opt_w) {
  ## End date is "yesterday"
  my ($end_year, $end_month, $end_day) =
    Add_Delta_Days($year + 1900, $mon + 1, $mday, -1);

  ## Start date is first day of the week
  my ($start_year, $start_month, $start_day) =
    Add_Delta_Days($end_year, $end_month, $end_day, -7);

  invoke_statistics($start_year, $start_month, $start_day, 
                    $end_year, $end_month, $end_day,
                    $flags);
}

if ($opt_d) {
  ## End date is "yesterday"
  my ($end_year, $end_month, $end_day) =
    Add_Delta_Days($year + 1900, $mon + 1, $mday, -1);

  ## Start date is "yesterday" too!
  my ($start_year, $start_month, $start_day) =
    ($end_year, $end_month, $end_day);

  invoke_statistics($start_year, $start_month, $start_day, 
                    $end_year, $end_month, $end_day,
                    $flags);
}

sub invoke_statistics {
  my ($sy, $sm, $sd, $ey, $em, $ed, $flags) = @_;
  my $sdate = sprintf("%0.4d-%0.2d-%0.2d", $sy, $sm, $sd);
  my $edate = sprintf("%0.4d-%0.2d-%0.2d", $ey, $em, $ed);
  my @args = ( $command, $queuename, $sdate, $edate, $flags );
  #print "/usr/local/bin/statdump.pl $flags $queuename $sdate $edate\n";
  system(@args) == 0 or die "system @args failed: $?"
}

sub usage {
  print STDERR "Usage: statcron.pl {-m|-w|-d} [ queuename ]\n";
}


