#!/usr/bin/perl

use strict;

if (!defined($ARGV[0])) {
  print "Usage: statdump.pl <queue name>|All [<days>|<date from>] [<date to>]\n";
  print "       <days> is the number of days ago to report from.\n";
  print "       <date from> and <date to> are in the format yyyy-mm-dd\n";
  exit 1;
}

use Date::Calc;
use Date::Parse;
use DBI;

my $database = "rt3";
my $hostname = "";
my $port = '';
my $user = "rt_user";
# Set this to your RT_User DB Password
my $password = "rt_pass";

my ($dayto, $dayfrom, $dayrange, $dayinterval);
my $dsn = "DBI:mysql:database=$database;host=$hostname;port=$port";
my $dbh = DBI->connect($dsn, $user, $password, {'RaiseError' => 1});

if (defined($ARGV[1])) {
  if ($ARGV[1] =~ /\-/) {
    $dayfrom = "\'$ARGV[1]\'";
    $dayrange = "from $dayfrom to ";
    if (defined($ARGV[2])) {
      $dayto = "\'$ARGV[2]\'";
      $dayrange .= $dayto;
    } else {
      my $query = "SELECT CURDATE();";
      my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
      my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
      while (my @ary = $sth->fetchrow_array) {
	    $dayto = $ary[0];
      }
      $dayrange .= "today";
    }
  } else {
    $dayinterval = $ARGV[1];
    $dayrange = "in the last $dayinterval days";
      my $query = "SELECT DATE_SUB(CURDATE(), INTERVAL $dayinterval DAY)";
      my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
      my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
      while (my @ary = $sth->fetchrow_array) {
	    $dayfrom = $ary[0]
      }
      my $query = "SELECT CURDATE()";
      my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
      my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
      while (my @ary = $sth->fetchrow_array) {
	    $dayto = $ary[0];
      }
  }
} else {
  $dayinterval = 7;
  $dayrange = "in the last $dayinterval days";
      my $query = "SELECT DATE_SUB(CURDATE(), INTERVAL $dayinterval DAY)";
      my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
      my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
      while (my @ary = $sth->fetchrow_array) {
  	$dayfrom = $ary[0];
      }
      my $query = "SELECT CURDATE()";
      my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
      my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
      while (my @ary = $sth->fetchrow_array) {
  	$dayto = $ary[0];
      }
}
	

my $createdtotal;
my $resolvedtotal;
my $queuequery;
my $thisqueue;

if ($ARGV[0] =~ /all/i) {
  $queuequery = "";
  $thisqueue = "all queues";
} else {
  my $query = "SELECT Id FROM Queues WHERE name=\"$ARGV[0]\"";
  my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
  my $rc = $sth->execute or die "Can't execute statement: $DBI::errstr";
  my $queueid;
  while (my @ary = $sth->fetchrow_array) {
    $queueid = $ary[0];
  }
  $queuequery = "Tickets.queue=$queueid and";
  $thisqueue = "the queue \"" . ucfirst $ARGV[0] . "\"";
}

my @prios = ( [90,1000], [70,89], [50,69], [10,49], [0,9] );
my @prioNames = ( "Urgent", "High", "Medium", "Low", "None" );

if ($dayfrom =~ /-/) { $dayfrom = "\'" . dequote($dayfrom) . "
00:00:00\'"; }
if ($dayto =~ /-/) { $dayto = "\'" . dequote($dayto) . " 23:59:59\'"; }

open_tickets($dayfrom);
tickets_opened($dayfrom, $dayto);
tickets_closed($dayfrom, $dayto);
open_tickets($dayto);
average_age($dayfrom, $dayto);
days_to_close($dayfrom, $dayto);

sub tickets_opened {
  my $dayfrom = shift;
  my $dayto = shift;
  my %t = ();
  my %topened = ();
  my $openedall = 0;

  print "\n\n";
  print "Tickets Opened\n";
  print "==============\n";
  my $query = "SELECT id,priority FROM Tickets WHERE
    $queuequery
    effectiveid = id
    AND status != 'deleted'
    AND created >= ($dayfrom) and created <= ($dayto);";

  #print "query: $query\n";

  my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
  my $rc = $sth->execute
      or die "Can't execute statement: $DBI::errstr";
  while (my @ary = $sth->fetchrow_array) {
    my $tid = $ary[0];
    my $priority = $ary[1];
    my $tp = 0;
    while ($priority < $prios[$tp][0]) {
      $tp++;
    }
    if ($tp >= @prios) { $tp = @prios - 1; }
    if ($t{$tid} != "") {
      print "Duplicate open of $tid\n";
    } else {
      $t{$tid} = $tp;
      $topened{$tp}++;
      #print "OPENED: $tid\n";
      $openedall++;
    }
  }
  for (my $tp = 0; $tp < @prios; $tp++) {
    print "$prioNames[$tp]\t$topened{$tp}\n";
  }
  print "ALL\t$openedall\n";
}

sub tickets_closed {
  my $dayfrom = shift;
  my $dayto = shift;
  my %t = ();
  my %tclosed = ();
  my $closedall = 0;

  print "\n\n";
  print "Tickets Closed\n";
  print "==============\n";
  my $query = "SELECT id,priority FROM Tickets WHERE
    $queuequery
    effectiveid = id
    AND (status = 'rejected' OR status = 'resolved')
    AND resolved <= $dayto
    AND resolved >= $dayfrom;";
  #print "query: $query\n";
  my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
  my $rc = $sth->execute
      or die "Can't execute statement: $DBI::errstr";
  while (my @ary = $sth->fetchrow_array) {
    my $tid = $ary[0];
    my $priority = $ary[1];
    my $tp = 0;
    while ($priority < $prios[$tp][0]) {
      $tp++;
    }
    if ($tp >= @prios) { $tp = @prios - 1; }
    if ($t{$tid} != "") {
      print "Duplicate closure of $tid\n";
    } else {
      $t{$tid} = $tp;
      $tclosed{$tp}++;
      #print "CLOSED: $tid\n";
      $closedall++;
    }
  }
  for (my $tp = 0; $tp < @prios; $tp++) {
    print "$prioNames[$tp]\t$tclosed{$tp}\n";
  }
  print "ALL\t$closedall\n";
}

sub days_to_close {
  my $dayfrom = shift;
  my $dayto = shift;
  my %numtickets = ();
  my %timetickets = ();
  my %t = ();
  my $aat;
  my $at;
  my $tp;

  print "\n\n";
  print "Days to Close\n";
  print "=============\n";
  my $nvwhere = "(NewValue='resolved' OR NewValue='rejected')";
  my $query = "
    SELECT Tickets.id,Transactions.Created FROM Transactions, Tickets WHERE
    $queuequery
    Tickets.effectiveid = Tickets.id
    AND Tickets.id=Transactions.ObjectId AND Transactions.Created >= $dayfrom and Transactions.Created <= $dayto AND $queuequery $nvwhere;";
    my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
    my $rc = $sth->execute
        or die "Can't execute statement: $DBI::errstr";
    while (my @ary = $sth->fetchrow_array) {
        $t{$ary[0]} = $ary[1];
    }
 
  foreach my $tid (keys %t) {
    my $closetime = $t{$tid};
    my @ary;
    my $query = "select created,Priority from Tickets where Id=$tid;";
    $sth = $dbh->prepare($query) or die "Can't prepare queue query";
    $rc = $sth->execute
        or die "Can't execute statement: $DBI::errstr";
    @ary = $sth->fetchrow_array;
    my $createtime = $ary[0];
    my $priority = $ary[1];
    $tp = 0;
    while ($priority < $prios[$tp][0]) {
      $tp++;
    }
    if ($tp >= @prios) { $tp = @prios - 1; }
    my $crs = str2time($createtime);
    my $cls = str2time($closetime);
    my $delta = $cls - $crs;
    $numtickets{$tp}++;
    $timetickets{$tp} += $delta;
  }
  
  my $ttickets = 0;
  my $ttime = 0;
  for ($tp = 0; $tp < @prios; $tp++) {
    if ($numtickets{$tp} != 0) {
      $at = $timetickets{$tp}/$numtickets{$tp};
    } else {
      $at = 0;
    }
    $ttickets += $numtickets{$tp};
    $ttime += $timetickets{$tp};
    printf ("%s\t%5.2f\n", $prioNames[$tp], (($at/60)/60)/24);
  }
  if ($ttickets != 0) {
    $aat = $ttime/$ttickets;
  } else {
    $aat = 0;
  }
  printf ("%s\t%5.2f\n", "ALL", (($aat/60)/60)/24); }  

sub average_age {
  my $dayfrom = shift;
  my $dayto = shift;
  my $tp;
  my $tid;
  my $aat;
  my %ticketp = ();
  my %tickett = ();
  my %ttp = ();
  my %tcount = ();
  my $allcount = 0;
  my $alltime = 0;
  my $delta = 0;
  my $at = 0;
  my @ary;

  print "\n\n";
  print "Average Age\n";
  print "===========\n";
  my $query = "
    select id,priority,created,resolved,status from Tickets where 
      $queuequery
      effectiveid = id
      AND NOT ((status='resolved' OR status='deleted' OR
status='rejected') AND (resolved < $dayfrom))
      AND created <= ($dayto);";
  
  #print "query: $query\n";

  my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
  my $rc = $sth->execute
      or die "Can't execute statement: $DBI::errstr";
  while (@ary = $sth->fetchrow_array) {
    $tid = $ary[0]; 
    my $priority = $ary[1]; 
    my $created = $ary[2]; 
    my $resolved = $ary[3]; 
    my $status = $ary[4]; 
    my $tp = 0;
    while ($priority < $prios[$tp][0]) {
      $tp++;
    }
    my $opendate = $created;
    my $enddate = $resolved;
    if ((str2time($resolved) > str2time(dequote($dayto))) ||
        ($resolved eq "1970-01-01 00:00:00")) {
      $enddate = dequote($dayto);
    }
    $delta = str2time($enddate) - str2time($opendate);
    if ($delta < 0) {
      #print "$tid\t$priority/$tp\t$created\t$resolved\t$status\t($delta)\n";
      $delta = 0;
    }
    $ticketp{$tid} = $tp;
    $tickett{$tid} = $delta;
  }
  
  foreach $tid (keys %ticketp) {
    my $tp = $ticketp{$tid};
    my $delta = $tickett{$tid};
    $ttp{$tp} += $delta;
    $tcount{$tp}++;
    $allcount++;
    $alltime += $delta;
  }
  
  my $ttickets = 0;
  my  $ttime = 0;
  for ($tp = 0; $tp < @prios; $tp++) {
    if ($tcount{$tp} != 0) {
      $at = $ttp{$tp}/$tcount{$tp};
    } else {
      $at = 0;
    }
    printf ("%s\t%5.2f\n", $prioNames[$tp], (($at/60)/60)/24);
  }
  if ($allcount != 0) {
    $aat = $alltime/$allcount;
  } else {
    $aat = 0;
  }
  printf ("%s\t%5.2f\n", "ALL", (($aat/60)/60)/24); }
  
sub open_tickets {
  my $asofdate = shift;
  my %ticketp = ();
  my %tcount = ();
  my @ary = ();
  my $allcount = 0;
  my $tp;
  my $tid;

  print "\n\n";
  print "Open Tickets as of " . dequote($asofdate) . "\n";
  print "======================================\n";
  my $query = "
    select id,priority from Tickets where 
    $queuequery
    effectiveid = id
    AND status != 'deleted'
    AND created <= $asofdate
    AND (
      (status='open' OR status='stalled' OR status='new')
      OR (
        NOT (status='open' OR status='stalled' OR status='new')
        AND resolved > $asofdate
      )
    );";
  #print "query: $query\n";

  my $sth = $dbh->prepare($query) or die "Can't prepare queue query";
  my $rc = $sth->execute
      or die "Can't execute statement: $DBI::errstr";
  while (@ary = $sth->fetchrow_array) {
    my $tid = $ary[0]; 
    my $priority = $ary[1]; 
    $tp = 0;
    while ($priority < $prios[$tp][0]) {
      $tp++;
    }
    if ($ticketp{$tid} != "") {
      print "Found duplicate ticket entry for $tid tp==$tp\n";
    }
    $ticketp{$tid} = $tp;
    #print "OPEN AS OF " . dequote($asofdate) . ": $tid\n";
  }

  foreach $tid (keys %ticketp) {
    $tp = $ticketp{$tid};
    $tcount{$tp}++;
    $allcount++;
  }

  for ($tp = 0; $tp < @prios; $tp++) {
    print "$prioNames[$tp]\t$tcount{$tp}\n";
  }
  print "ALL\t$allcount\n";
}

sub dequote {
  my $string = shift;
  my $temp = $string;

  $temp =~ s/\'//g;
  return $temp;
}
